/**CHeaderFile*****************************************************************

  FileName    [mc.h]

  PackageName [mc]

  Synopsis    [Fair CTL model checking algorithms. External header file.]

  Description [Fair CTL model checking algorithms. External header file.]

  Author      [Marco Roveri]

  Revision    [$Id: $]

  Copyright   [ Copyright (c) 1998 by ITC-IRST and Carnegie Mellon University.
  All Rights Reserved. This software is for educational purposes only.
  Permission is given to academic institutions to use, copy, and modify
  this software and its documentation provided that this introductory
  message is not removed, that this software and its documentation is
  used for the institutions' internal research and educational purposes,
  and that no monies are exchanged. No guarantee is expressed or implied
  by the distribution of this code.
  Send bug-reports and/or questions to: nusmv@irst.itc.it. ]

******************************************************************************/

#ifndef _MC
#define _MC

#include "util.h"
#include "dd.h"
#include "opt.h"

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/

EXTERN void Mc_Init ARGS((void));
EXTERN void Mc_End ARGS((void));
EXTERN void check_spec ARGS((node_ptr));
EXTERN void check_compute ARGS((node_ptr));
EXTERN void check_transition_relation ARGS((void));
EXTERN void check_invar ARGS((node_ptr));
EXTERN void check_invar_fb ARGS((node_ptr));
EXTERN void check_invar_strong ARGS((node_ptr));
EXTERN void check_after ARGS((node_ptr));
EXTERN void print_spec  ARGS((FILE *file, node_ptr n));
EXTERN void print_invar ARGS((FILE *file, node_ptr n));
EXTERN void print_compute ARGS((FILE *, node_ptr));
EXTERN void reset_trace_number ARGS((void));
EXTERN bdd_ptr  compute_fair_states      ARGS((void));
EXTERN void     compute_reachable_states ARGS((void));
EXTERN bdd_ptr  ex      ARGS((bdd_ptr));
EXTERN bdd_ptr  ef      ARGS((bdd_ptr));
EXTERN bdd_ptr  eg      ARGS((bdd_ptr));
EXTERN bdd_ptr  eu      ARGS((bdd_ptr, bdd_ptr));
EXTERN bdd_ptr  au      ARGS((bdd_ptr, bdd_ptr));
EXTERN bdd_ptr  ebu     ARGS((bdd_ptr, bdd_ptr, int, int));
EXTERN bdd_ptr  abu     ARGS((bdd_ptr, bdd_ptr, int, int));
EXTERN bdd_ptr  ebf     ARGS((bdd_ptr, int, int));
EXTERN bdd_ptr  ebg     ARGS((bdd_ptr, int, int));
EXTERN int      minu    ARGS((bdd_ptr, bdd_ptr));
EXTERN int      maxu    ARGS((bdd_ptr, bdd_ptr));
EXTERN node_ptr explain ARGS((node_ptr, node_ptr, node_ptr));
EXTERN node_ptr ex_explain ARGS((node_ptr p, bdd_ptr f));
EXTERN bdd_ptr  eval_spec      ARGS((node_ptr, node_ptr));
EXTERN node_ptr eval_formula_list ARGS((node_ptr, node_ptr));
EXTERN void     free_formula_list ARGS((DdManager * dd, node_ptr formula_list));
EXTERN int      eval_compute   ARGS((node_ptr, node_ptr));
EXTERN int      check_AG_only  ARGS((node_ptr, node_ptr));
EXTERN node_ptr make_AG_counterexample ARGS((node_ptr l, bdd_ptr s0));
EXTERN void     compute_plan                ARGS((bdd_ptr, node_ptr, int, int));
EXTERN void     check_invariant_forward     ARGS((node_ptr));
EXTERN void     check_invariant_forward_opt ARGS((node_ptr));
EXTERN void     compute_plan_fb             ARGS((bdd_ptr, bdd_ptr, node_ptr, int, bdd_ptr, node_ptr, int));
EXTERN void     temporal_projection         ARGS((node_ptr, node_ptr));
EXTERN void     eval_after                  ARGS((node_ptr, bdd_ptr *));
EXTERN void     check_invariant_strong      ARGS((node_ptr));
EXTERN void     check_invariant_forward_backward    ARGS((node_ptr));
EXTERN void     check_invariant_forward_backward_op ARGS((node_ptr));
EXTERN bdd_ptr  trans_is_total                      ARGS((void));
EXTERN void     print_explanation           ARGS((node_ptr));
EXTERN void     print_reachable_states      ARGS((void));
EXTERN void     init_traces_hash      ARGS((void));
EXTERN void     insert_traces_hash    ARGS((node_ptr, node_ptr));
EXTERN node_ptr lookup_traces_hash    ARGS((node_ptr));
EXTERN void     clear_traces_hash     ARGS((void));
EXTERN boolean  current_state_bdd_exist   ARGS((void));
EXTERN boolean  current_state_label_exist ARGS((void));
EXTERN boolean  current_state_exist       ARGS((void));
EXTERN bdd_ptr  current_state_bdd_get     ARGS((void));
EXTERN node_ptr current_state_label_get   ARGS((void)); 
EXTERN void     inc_trace_number          ARGS((void));
EXTERN void     dec_trace_number          ARGS((void));
EXTERN int      get_trace_number          ARGS((void));
EXTERN void     current_state_bdd_free    ARGS((void));
EXTERN void     current_state_label_reset ARGS((void));
EXTERN void     current_state_bdd_reset   ARGS((void));
EXTERN void     current_state_bdd_set     ARGS((bdd_ptr));
EXTERN void     current_state_set         ARGS((bdd_ptr, node_ptr));
EXTERN void     current_state_label_set   ARGS((node_ptr));

#endif /* _MC */
